/*
             Copyright 1994,1995 Digital Equipment Corporation
	     Copyright 2000 Compaq Computer Corporation
                        All rights reserved.

   Redistribution and use in source and binary forms are
   permitted provided that: 1) source distributions retain this
   entire copyright notice and 2) distributions including
   binaries display the following acknowledgment in its
   documentation:

           "This product includes software developed by the
            Digital Equipment Corporation and Compaq Computer
	    Corporation"
*/

#include <stdio.h>

#include <X11/Xos.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xmu/Xmu.h>
#include "vmubc.h"
#include "system.h"

char *progname;
struct def_t defs;
extern XtIntervalId timer;

int interval = 1;
int firstrun = 1; // MKN: Added for first-pass initialization
int bars_counted = 0; // MKN: Added for tty output formatting
int count = 0;
int log_to_tty = FALSE;
int initialize = TRUE;
int do_bar[MAX_BARS]; // MKN: Use MAX_BARS

void
usage()
{
/*
** MKN: Remove UBC_status. Move info into UBC_page.
** MKN: Added "-ttyx". Minor cosmetic change to "usage. 13-Nov-2000
** MKN: Added "-syscall". 23-Oct-2000
** MKN: Added "-intr". 20-Oct-2000
*/
    (void) fprintf(stderr, "\
Usage: %s [Xt options]\n\t[-h/-help]\n\t[-count iterations]\n\t[-interval delay_seconds] \
\n\t[-tty][-ttyx]\n\t[-cpu]\n\t[-memory]\n\t[-ubc]\n\t[-metadata]\n\t[-pagefault] \
\n\t[-swap]\n\t[-intr]\n\t[-syscall]\n",
     progname);
    exit(-1);
}

char *
xmalloc(n)
int n;
{
    extern char *malloc();
    char *cp = malloc((unsigned) n);
    if (cp == NULL) {
	    (void) fprintf(stderr, "Couldn't malloc %d bytes\n", n);
	    exit(-1);
    }
    return cp;
}

static int nbars;

void parseopts( argc, argv )
int argc;
char **argv;
{
    int i;
    int read_int;
    int *int_addr;
    int do_all = TRUE;

    for (i = 0; i < nbars; i++)
	do_bar[i] = FALSE;

    /* get the name of the program (goes into the title bar) */

    if ((progname = strrchr(argv[0], '/')) == NULL)
	progname = argv[0];
    else
	progname++;

    /* look for other options */
    
    read_int = FALSE;
    for (i =1; i < argc; i++) {
	if ( read_int ) {
	    if(sscanf( argv[i], "%d", int_addr ) != 1)
		usage();
	    read_int = FALSE;
	}
	else if (!strcmp( argv[i], "-interval" ) ) {
	    read_int = TRUE;
	    int_addr = &interval;
	}
	else if (!strcmp( argv[i], "-count" ) ) {
	    read_int = TRUE;
	    int_addr = &count;
	}
	else if (!strcmp( argv[i], "-cpu")) {
	   do_all = FALSE;
	   do_bar[0] = TRUE;
	}
	else if (!strcmp( argv[i], "-memory")) {
	   do_all = FALSE;
	   do_bar[1] = TRUE;
	}
	else if (!strcmp( argv[i], "-ubc")) {
	   do_all = FALSE;
	   do_bar[2] = TRUE;
	   do_bar[3] = TRUE;
	}
	else if (!strcmp( argv[i], "-metadata")) {
	   do_all = FALSE;
	   do_bar[4] = TRUE;
	}
	else if (!strcmp( argv[i], "-pagefault")) {
	   do_all = FALSE;
	   do_bar[5] = TRUE;
	}
	else if (!strcmp( argv[i], "-swap")) {
	   do_all = FALSE;
	   do_bar[6] = TRUE;
	}
/*
** MKN: Added "-intr". 20-Oct-2000
*/
	else if (!strcmp( argv[i], "-intr")) {
	   do_all = FALSE;
	   do_bar[7] = TRUE;
	}
/*
** MKN: Added "-syscall". 20-Oct-2000
*/
	else if (!strcmp( argv[i], "-syscall")) {
	   do_all = FALSE;
	   do_bar[8] = TRUE;
	}
	else if (!strcmp( argv[i], "-ttyx")) {
	    do_all = FALSE;
	    do_bar[0] = TRUE;
	    do_bar[1] = TRUE;
	    do_bar[5] = TRUE;
	    do_bar[6] = TRUE;
	    log_to_tty = TRUE;
	}
	else if (!strcmp( argv[i], "-tty")) {
	    log_to_tty = TRUE;
	}
	else if (!strcmp( argv[i], "-h")) {
	    usage();
	}
	else if (!strcmp( argv[i], "-help")) {
	    usage();
	}
    }

    if (read_int)
	usage();

    if (interval < 1)
	interval = 1;

    if (do_all)
	for (i = 0; i < nbars; i++)
	    do_bar[i] = TRUE;

/*
** MKN: Count actual number of bars
*/
    for (i = 0; i < nbars; i++)
      if (do_bar[i] == TRUE) bars_counted++;

}

    
main(argc, argv)
int argc;
char **argv;
{
    int i;
    extern char *strchr(/* const char *, char */);


    nbars = num_bars();
    parseopts( argc, argv );

    if (nbars == 0) {
	(void) fprintf(stderr, "num_bars returned 0 - something is wrong\n");
	exit(-1);
    }

    init_bars();

    if (log_to_tty)
	init_for_tty();
    else
        create_widgets( progname, argc, argv );

    /* Do the initial update */
    display_bars( nbars );
    initialize = FALSE;

    if (log_to_tty) {
        fprintf(stdout,"\n");
	if (count == 0) {
	    while( TRUE ) {
	    write_tty_header();
		for (i = 0; i < 24; i++) {
		    display_bars( nbars );
		    fprintf(stdout,"\n");
		    sleep(interval);
		}
	    }
	}
	else {
	    for (i = 0; count > 0; i=(i+1)%24, count--) {
		if (i == 0)
		    write_tty_header();
		    display_bars( nbars );
		    fprintf(stdout,"\n");
		    sleep(interval);
	    }
	}
    }
    else {
	do_widget_loop();
    }
}



